"use strict";

var Fashion = require('../Base.js');
var Type = require('./Type.js');
var Text = require('./Text.js');
var Numeric = require('./Numeric.js');

var Bool = require('./Bool.js'),
    True = Bool.True,
    False = Bool.False;

var Literal = require('./Literal.js'),
    Null = Literal.Null;

module.exports = {
    unboxType (expression) {
        var val = expression;
        if (val && val.$isFashionType && val.$canUnbox) {
            val = val.value;
            if (expression.$isFashionString || expression.$isFashionLiteral) {
                if (val === 'none' || val === 'null') {
                    val = null;
                }
            } else if (expression.$isFashionList) {
                val = expression.items;
            }
        }
        return val;
    },

    boxType(expression) {
        if (expression && expression.$isFashionType) {
            return expression;
        }

        if (expression == null) { // null || undefined
            return Null;
        }
        if (expression === true) {
            return True;
        }
        if (expression === false) {
            return False;
        }

        var typeOf = typeof expression;
        switch (typeOf) {
            case 'string':
                return new Text(expression);
            case 'number':
                return new Numeric(expression);
            default:
                break;
        }

        return expression;
    }

};

Fashion.apply(Type, {
    box: module.exports.boxType,
    unbox: module.exports.unboxType
});