"use strict";

var Literal = require('../export/type/Literal.js');
var Bool = require('../export/type/Bool.js');

module.exports = {
    init(runtime) {
        runtime.register({
            __if: function (condition, if_true, if_false) {
                return this.unbox(condition) ? if_true : if_false;
            },

            theme_image: function (theme, file) {
                return new Literal('url("resources/images/' + theme.value + '/' + file.value + '")');
            },

            prefixed: function (prefix, items) {
                if (arguments.length > 2) {
                    items = Array.prototype.slice.call(arguments, 1);
                }

                prefix = this.unbox(prefix);

                if (prefix.indexOf("-") === 0) {
                    prefix = prefix.substring(1);
                }
                if (!Array.isArray(items)) {
                    if (items.supports && items.supports(prefix)) {
                        return Bool.True;
                    }
                } else {
                    var ln = items.length, i, arg;
                    for (i = 0; i < ln; i++) {
                        arg = items[i];
                        if (arg.supports && arg.supports(prefix)) {
                            return Bool.True;
                        }
                    }
                }

                return Bool.False;
            },

            _owg: function (value) {
                return new Literal(value.toPrefixedString('owg'));
            },

            _webkit: function (value) {
                return new Literal(value.toPrefixedString('-webkit-'));
            },

            _o: function (value) {
                return new Literal(value.toPrefixedString('-o-'));
            },

            _moz: function (value) {
                return new Literal(value.toPrefixedString('-moz-'));
            }
        });
    }

}
